@@ -17,7 +17,7 @@
     decoded into an unicode object if possible and if it makes sense.
 
 
-    :copyright: 2007-2008 by Armin Ronacher, Georg Brandl.
+    :copyright: (c) 2009 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
 import tempfile
@@ -212,9 +212,9 @@ class BaseRequest(object):
         members:
 
         - `filename` - The name of the uploaded file, as a Python string.
-        - `type` - The content type of the uploaded file.
-        - `data` - The raw content of the uploaded file.
+        - `content_type` - The content type of the uploaded file.
         - `read()` - Read from the stream.
+        - `save()` - Save the upload into a file or file pointer.
 
         Note that `files` will only contain data if the request method was POST
         and the ``<form>`` that posted to the request had
